import dataclasses
import os
from pathlib import Path
from typing import List

from jig.collector.domain.ast import ImportFrom
from jig.collector.domain.values.import_path import ImportPath
from jig.collector.domain.values.module_path import ModulePath


@dataclasses.dataclass(frozen=True)
class SourceFilePath:
    root_path: Path
    file_path: Path

    @property
    def module_path(self) -> ModulePath:
        relative_path = self.relative_path_from_root

        # root直下のファイルの場合、ファイル名がそのままモジュール名
        # そこからparentを辿ろうとするとパス表現が壊れる（"."になる）ため、root直下の場合は別で処理する
        if str(relative_path) == relative_path.name:
            return ModulePath(names=[relative_path.stem])

        # ディレクトリ配下の場合はパッケージファイル（__init__.py）の可能性もあるのでそれを考慮する
        dirname = relative_path.parent

        names = str(dirname).split(os.sep)
        if relative_path.stem != "__init__":
            names.append(relative_path.stem)

        return ModulePath(names=names)

    @property
    def filename(self) -> str:
        return self.file_path.name

    def module_path_with_level(self, level: int) -> ModulePath:
        """
        levelの数だけ上位に上ったパスを返します。
        :param level:
        :return:
        """
        assert level >= 1

        parent = self.relative_path_from_root.parents[level - 1]

        names = str(parent).split(os.sep)

        return ModulePath(names=names)

    def import_from_to_import_paths(self, import_from: ImportFrom) -> List[ImportPath]:
        """
        このソースファイルパスを基準にimport from を解決し、ModulePathのリストを返します。
        :param import_from:
        :return:
        """
        level = import_from.level if import_from.level is not None else 0

        if level < 1:
            assert import_from.module is not None

            p = ImportPath.from_str(import_from.module)
            return [p.join(alias.name) for alias in import_from.names]

        module_path = self.module_path_with_level(level)
        p = ImportPath.from_str(str(module_path))

        if import_from.module:
            p += ImportPath.from_str(import_from.module)

        return [p.join(alias.name) for alias in import_from.names]

    @property
    def relative_path_from_root(self) -> Path:
        # Path.relative_to は文字列化したパスから算出されるので絶対パスに揃えてから利用する
        root_path = self.root_path.absolute()
        file_path = self.file_path.absolute()

        return file_path.relative_to(root_path)

    @property
    def is_package(self):
        return self.file_path.stem == "__init__"



import dataclasses
from typing import List, Dict, Optional

from jig.analyzer.domain.dependency.module_dependency import ModuleDependency
from jig.collector.domain.source_code.source_code_collection import SourceCodeCollection
from jig.collector.domain.source_code.source_code_import_dependency import (
    SourceCodeImportDependency,
)
from jig.collector.domain.values.import_path import ImportPath
from jig.collector.domain.values.module_path import ModulePath


@dataclasses.dataclass(frozen=True)
class ImportDependencyCollection:
    _dependencies: Dict[str, SourceCodeImportDependency]

    @classmethod
    def build(
        cls, dependencies: List[SourceCodeImportDependency]
    ) -> "ImportDependencyCollection":
        return cls(
            _dependencies={
                str(dependency.source_module_path): dependency
                for dependency in dependencies
            }
        )

    @classmethod
    def build_from_source_code_collection(
        cls, source_code_collection: SourceCodeCollection
    ) -> "ImportDependencyCollection":
        return cls.build(
            dependencies=[
                source_code.build_import_dependency()
                for source_code in source_code_collection
            ]
        )

    def get_by_module_path(
        self, module_path: ModulePath
    ) -> Optional[SourceCodeImportDependency]:
        return self._dependencies.get(str(module_path))

    def build_module_dependencies(self) -> List[ModuleDependency]:
        """
        selfで保持しているModulePathからImportPathへの依存情報を
        ModulePathからModulePathへの依存情報に変換する
        """
        dependencies = []
        for dep in self._dependencies.values():
            src = dep.source_module_path
            for dest_import_path in dep.import_paths:
                dest_module_path = self.detect_module_path(import_path=dest_import_path)
                if dest_module_path:
                    dependencies.append(
                        ModuleDependency(src=src, dest=dest_module_path)
                    )

        return dependencies

    def detect_module_path(self, import_path: ImportPath) -> Optional[ModulePath]:
        module_path = ModulePath.from_str(str(import_path))
        if self.get_by_module_path(module_path):
            return module_path

        if module_path.depth > 1:
            parent_path = module_path.parent()
            if self.get_by_module_path(parent_path):
                return parent_path

        return None



import dataclasses

from graphviz import Digraph

from jig.visualizer.module_dependency.domain.model.graph import Graph
from .cluster_renderer import ClusterRenderer


@dataclasses.dataclass
class GraphRenderer:
    graph: Graph

    def render(self) -> Digraph:
        d = Digraph()

        for node in sorted(self.graph.nodes):
            d.node(name=node.path.name, **node.style.to_dict())

        for edge in sorted(self.graph.edges):
            d.edge(edge.tail.name, edge.head.name, **edge.style.to_dict())

        for cluster in self.graph.clusters.values():
            cluster_renderer = ClusterRenderer(cluster)
            d.subgraph(cluster_renderer.render())

        return d



import dataclasses
from typing import Optional, List

from typed_ast import ast3 as ast


@dataclasses.dataclass(frozen=True)
class ClassDef:
    name: str
    _ast: ast.ClassDef = dataclasses.field(repr=False, compare=False)


@dataclasses.dataclass(frozen=True)
class Alias:
    name: str
    asname: Optional[str]


@dataclasses.dataclass(frozen=True)
class Import:
    names: List[Alias]
    _ast: ast.Import = dataclasses.field(repr=False, compare=False)

    @classmethod
    def from_ast(cls, import_ast: ast.Import) -> "Import":
        names = [
            Alias(name=name_alias.name, asname=name_alias.asname)
            for name_alias in import_ast.names
        ]

        return cls(names=names, _ast=import_ast)


@dataclasses.dataclass(frozen=True)
class ImportFrom:
    """
    module: fromで設定されるモジュール名。'from . import xxxx' の場合はNone
    names: importリスト
    level: fromのインポートレベル。'from os' => 0, 'from .' => 1, 'from ..' => 2
           levelがNoneになる条件は不明だが、ドキュメントの定義に合わせてOptional型で定義している
    """

    module: Optional[str]
    names: List[Alias]
    level: Optional[int]
    _ast: ast.ImportFrom = dataclasses.field(repr=False, compare=False)

    @classmethod
    def from_ast(cls, import_from: ast.ImportFrom) -> "ImportFrom":
        names = [
            Alias(name=alias.name, asname=alias.asname) for alias in import_from.names
        ]

        return cls(
            module=import_from.module,
            names=names,
            level=import_from.level,
            _ast=import_from,
        )


@dataclasses.dataclass(frozen=True)
class JigAST:
    _ast: ast.AST

    @classmethod
    def parse(cls, source: str, filename: str = "<unknown>") -> "JigAST":
        tree = ast.parse(source=source, filename=filename)
        return cls(tree)

    def imports(self) -> List[Import]:
        visitor = ImportVisitor()
        visitor.visit(self._ast)

        return visitor.imports

    def import_froms(self) -> List[ImportFrom]:
        visitor = ImportFromVisitor()
        visitor.visit(self._ast)

        return visitor.import_froms

    def class_defs(self) -> List[ClassDef]:
        visitor = ClassDefVisitor()
        visitor.visit(self._ast)

        return visitor.class_defs


@dataclasses.dataclass(frozen=True)
class JigSourceCode:
    imports: List[Import]
    import_froms: List[ImportFrom]
    class_defs: List[ClassDef]

    @classmethod
    def build(cls, source: str, filename: str) -> "JigSourceCode":
        jig_ast = JigAST.parse(source=source, filename=filename)

        return cls(
            imports=jig_ast.imports(),
            import_froms=jig_ast.import_froms(),
            class_defs=jig_ast.class_defs(),
        )


@dataclasses.dataclass
class ClassDefVisitor(ast.NodeVisitor):
    class_defs: List[ClassDef] = dataclasses.field(default_factory=list)

    def visit_ClassDef(self, node):
        self.class_defs.append(ClassDef(name=node.name, _ast=node,))


@dataclasses.dataclass
class ImportVisitor(ast.NodeVisitor):
    imports: List[Import] = dataclasses.field(default_factory=list)

    def visit_Import(self, node):
        self.imports.append(Import.from_ast(node))


@dataclasses.dataclass
class ImportFromVisitor(ast.NodeVisitor):
    import_froms: List[ImportFrom] = dataclasses.field(default_factory=list)

    def visit_ImportFrom(self, node):
        self.import_froms.append(
            ImportFrom(
                module=node.module,
                names=[
                    Alias(name=name_alias.name, asname=name_alias.asname)
                    for name_alias in node.names
                ],
                level=node.level,
                _ast=node,
            )
        )



import dataclasses
from typing import List


@dataclasses.dataclass(frozen=True)
class ModulePath:
    name: str

    def __post_init__(self):
        # not empty
        assert self.name

    @property
    def path_level(self) -> int:
        return len(self.parts)

    @property
    def parts(self) -> List[str]:
        return self.name.split(".")

    def __lt__(self, other: "ModulePath"):
        return self.name < other.name

    def belongs_to(self, other: "ModulePath") -> bool:
        if self.path_level < other.path_level:
            return False

        return all(map(lambda pair: pair[0] == pair[1], zip(self.parts, other.parts)))

    def limit_path_level(self, max_path_level: int) -> "ModulePath":
        assert max_path_level > 0

        new_name = ".".join(self.name.split(".")[:max_path_level])
        return ModulePath(new_name)



import dataclasses
from typing import Optional, List, Tuple, Iterator

from jig.visualizer.module_dependency.domain.value.module_edge import (
    ModuleEdge,
    ModuleEdgeCollection,
)
from jig.visualizer.module_dependency.domain.value.module_node import ModuleNode


@dataclasses.dataclass
class MasterGraph:
    edges: ModuleEdgeCollection = dataclasses.field(
        default_factory=ModuleEdgeCollection
    )

    @classmethod
    def from_tuple_list(cls, edges: List[Tuple[str, str]]) -> "MasterGraph":
        return cls(ModuleEdgeCollection.from_tuple_list(edges))

    def find_parent_edge(self, edge: ModuleEdge) -> Optional[ModuleEdge]:
        return self.edges.find_parent_edge(edge=edge)

    def find_nodes(self, node: ModuleNode) -> List[ModuleNode]:
        result = []

        for new_edge in self.edges:
            if new_edge.head.belongs_to(node):
                result.append(new_edge.head)
            if new_edge.tail.belongs_to(node):
                result.append(new_edge.tail)

        return result

    def find_edges(self, node: ModuleNode) -> List[ModuleEdge]:
        result = []

        for new_edge in self.edges:
            if new_edge.head.belongs_to(node) and new_edge.tail.belongs_to(node):
                result.append(new_edge)

        return result

    def __iter__(self) -> Iterator[ModuleEdge]:
        return iter(self.edges)



import dataclasses
from typing import Dict, Optional

from .penwidth import Color, PenWidth
from .module_path import ModulePath


@dataclasses.dataclass(frozen=True)
class ModuleNodeStyle:
    color: Color = dataclasses.field(default=Color.Black)
    fontcolor: Color = dataclasses.field(default=Color.Black)
    penwidth: PenWidth = dataclasses.field(default=PenWidth.Normal)
    filled: bool = False
    invisible: bool = False

    def to_dict(self) -> Dict[str, str]:
        return {
            "color": self.color.value,
            "fontcolor": self.fontcolor.value,
            "penwidth": self.penwidth.to_size(self.penwidth),
            "style": self.style,
        }

    def _clone(self, **changes) -> "ModuleNodeStyle":
        return dataclasses.replace(self, **changes)

    def with_penwidth(self, penwidth: PenWidth) -> "ModuleNodeStyle":
        return self._clone(penwidth=penwidth)

    @property
    def style(self) -> str:
        if self.invisible:
            return "invisible"
        if self.filled:
            return "filled"

        return ""


@dataclasses.dataclass(frozen=True)
class ModuleNode:
    path: ModulePath
    style: ModuleNodeStyle = dataclasses.field(
        default_factory=ModuleNodeStyle, compare=False
    )

    @classmethod
    def from_str(cls, path: str) -> "ModuleNode":
        return cls(path=ModulePath(name=path))

    @classmethod
    def build(
        cls, path: ModulePath, style: Optional[ModuleNodeStyle] = None
    ) -> "ModuleNode":
        return cls(path=path, style=style or ModuleNodeStyle())

    @property
    def name(self) -> str:
        return self.path.name

    @property
    def path_level(self) -> int:
        return self.path.path_level

    def __lt__(self, other: "ModuleNode"):
        return self.path < other.path

    def belongs_to(self, other: "ModuleNode") -> bool:
        return self.path.belongs_to(other.path)

    def limit_path_level(self, max_path_level: int) -> "ModuleNode":
        assert max_path_level > 0

        new_path = self.path.limit_path_level(max_path_level)
        return ModuleNode(new_path)

    def to_invisible(self) -> "ModuleNode":
        return self.build(path=self.path, style=ModuleNodeStyle(invisible=True))

    def with_style(self, style: ModuleNodeStyle) -> "ModuleNode":
        return self.build(path=self.path, style=style)



import dataclasses
import os
from pathlib import Path
from typing import List

from jig.collector.domain.source_code.source_code_collection import SourceCodeCollection
from jig.collector.domain.source_code.source_code import SourceCode
from jig.collector.domain.source_file.source_file import SourceFile
from jig.collector.domain.source_file.source_file_path import SourceFilePath


@dataclasses.dataclass(frozen=True)
class SourceCodeCollector:
    root_path: Path

    def collect(self, target_path: Path) -> SourceCodeCollection:
        source_codes = []
        if target_path.is_dir():
            source_codes.extend(self.collect_directory(target_path))
        else:
            source_codes.append(self.collect_file(target_path))

        return SourceCodeCollection(source_codes)

    def collect_file(self, target_path: Path) -> SourceCode:
        source = target_path.read_text()

        return SourceCode.build(
            file=SourceFile(
                source_file_path=SourceFilePath(
                    root_path=self.root_path, file_path=target_path
                ),
                content=source,
                size=os.path.getsize(str(target_path)),
            ),
        )

    def collect_directory(self, target_path: Path) -> List[SourceCode]:
        result = []
        for cur_dir, dirs, files in os.walk(str(target_path)):
            for file in files:
                if not file.endswith(".py"):
                    continue

                path = Path(cur_dir).joinpath(file)

                result.append(self.collect_file(target_path=path))

        return result



import dataclasses
from typing import Union, List

from jig.collector.domain.values.import_path import ImportPath
from jig.collector.domain.values.import_path_collection import ImportPathCollection
from jig.collector.domain.values.module_path import ModulePath


@dataclasses.dataclass(frozen=True)
class SourceCodeImportDependency:
    source_module_path: ModulePath
    import_paths: ImportPathCollection

    @classmethod
    def build(
        cls,
        source_module_path: ModulePath,
        import_paths: Union[List[ImportPath], ImportPathCollection],
    ) -> "SourceCodeImportDependency":
        if not isinstance(import_paths, ImportPathCollection):
            import_paths = ImportPathCollection(import_paths)

        return cls(source_module_path=source_module_path, import_paths=import_paths)



import dataclasses
from typing import List


@dataclasses.dataclass(frozen=True)
class ModulePath:
    names: List[str]

    def __post_init__(self):
        if any([name.find(".") >= 0 for name in self.names]):
            raise ValueError(f"An invalid name specified in `{self.names}`")
        if not self.names:
            raise ValueError("names must have least one element.")

    @classmethod
    def build(cls, names: List[str]) -> "ModulePath":
        return cls(names)

    def __add__(self, other: "ModulePath") -> "ModulePath":
        return ModulePath(names=self.names + other.names)

    def join(self, name: str) -> "ModulePath":
        """
        パスを追加します
        :param name:
        :return:
        """
        return ModulePath(names=self.names + [name])

    @property
    def depth(self):
        return len(self.names)

    def path_in_depth(self, depth: int) -> "ModulePath":
        """
        このパスを指定されたdepthだけ辿ったパスを新たに返します
        :param depth:
        :return:
        """
        assert depth > 0
        return ModulePath(names=self.names[:depth])

    def parent(self) -> "ModulePath":
        return self.build(names=self.names[:-1])

    def belongs_to(self, other: "ModulePath") -> bool:
        """
        パスがotherに含まれているかどうかを返します。
        :param other:
        :return:
        """
        if self.depth < other.depth:
            return False

        return all([p1 == p2 for p1, p2 in zip(self.names, other.names)])

    def __str__(self) -> str:
        return ".".join(self.names)

    @classmethod
    def from_str(cls, path: str) -> "ModulePath":
        names = path.split(".")
        return cls(names=names)

    def match_module_names(self, module_names: List[str]) -> bool:
        return any(
            [self.match_module_name(module_name) for module_name in module_names]
        )

    def match_module_name(self, module_name: str) -> bool:
        path_str = str(self)
        return path_str == module_name or path_str.startswith(module_name + ".")



import dataclasses
from typing import Set, List, Dict, Optional, Union

from jig.visualizer.module_dependency.domain.model.master_graph import MasterGraph
from jig.visualizer.module_dependency.domain.value.cluster import Cluster
from jig.visualizer.module_dependency.domain.value.module_edge import (
    ModuleEdge,
    ModuleEdgeCollection,
    ModuleEdgeStyle,
)
from jig.visualizer.module_dependency.domain.value.module_node import (
    ModuleNode,
    ModuleNodeStyle,
)
from jig.visualizer.module_dependency.domain.value.penwidth import Color, PenWidth


@dataclasses.dataclass
class Graph:
    master_graph: MasterGraph = dataclasses.field(default_factory=MasterGraph)
    nodes: Set[ModuleNode] = dataclasses.field(default_factory=set)
    edges: Set[ModuleEdge] = dataclasses.field(default_factory=set)
    clusters: Dict[ModuleNode, Cluster] = dataclasses.field(default_factory=dict)

    def to_dict(self) -> dict:
        nodes = sorted([n.name for n in self.nodes])
        edges = sorted([(e.tail.name, e.head.name) for e in self.edges])
        clusters = dict(
            [(node.name, cluster.to_dict()) for node, cluster in self.clusters.items()]
        )

        return {"nodes": nodes, "edges": edges, "clusters": clusters}

    def find_node_owner(self, node: ModuleNode) -> Optional[Union["Graph", Cluster]]:
        for cluster in self.clusters.values():
            owner = cluster.find_node_owner(node)
            if owner:
                return owner

        if node in self.nodes:
            return self

        return None

    def add_node(self, node: ModuleNode):
        self.nodes.add(node)

    def add_edge(self, edge: ModuleEdge):
        if edge.is_self_loop():
            return

        self.edges.add(edge)

        self.add_node(edge.tail)
        self.add_node(edge.head)

    def add_cluster(self, cluster: Cluster):
        not_included = cluster.children - self.nodes
        if not_included:
            raise ValueError(f"Graphのnodesに含まれないchildrenがあります: {not_included}")

        self.clusters[cluster.node] = cluster

    def remove_node(self, node: ModuleNode):
        if node in self.nodes:
            self.nodes.remove(node)

        edges = list(filter(lambda e: e.has_node(node), self.edges))
        for edge in edges:
            self.edges.remove(edge)

        self._remove_node_from_cluster(node)

    def _remove_node_from_cluster(self, node: ModuleNode):
        # Dict#values() で for を回しているときには、要素削除できないので、 List にキャストする
        for cluster in list(self.clusters.values()):
            cluster.remove(node)

            if cluster.is_empty:
                del self.clusters[cluster.node]

    def hide_node(self, node: ModuleNode):
        if node in self.nodes:
            self.nodes.remove(node)
            self.nodes.add(node.to_invisible())

        edges = list(filter(lambda e: e.has_node(node), self.edges))
        for edge in edges:
            self.edges.remove(edge)
            self.edges.add(edge.to_invisible())

        for cluster in self.clusters.values():
            cluster.hide_node(node)

    def style(
        self, node: ModuleNode, color: Color, fontcolor: Color, penwidth: PenWidth
    ):
        node_style = ModuleNodeStyle(
            color=color, fontcolor=fontcolor, penwidth=penwidth
        )
        edge_style = ModuleEdgeStyle(
            color=color, fontcolor=fontcolor, penwidth=penwidth
        )

        if node in self.nodes:
            self.nodes.remove(node)
            self.nodes.add(node.with_style(style=node_style))

        edges = list(filter(lambda e: e.has_node(node), self.edges))
        for edge in edges:
            self.edges.remove(edge)
            self.edges.add(edge.with_style(style=edge_style))

    def auto_highlight(self):
        source_nodes: Dict[ModuleNode, List[ModuleEdge]] = dict(
            [(node, []) for node in self.nodes]
        )
        dest_nodes: Dict[ModuleNode, List[ModuleEdge]] = dict(
            [(node, []) for node in self.nodes]
        )
        for edge in self.edges:
            source_nodes[edge.tail].append(edge)
            dest_nodes[edge.head].append(edge)

        # entrypoint
        entrypoint_node_style = ModuleNodeStyle(
            color=Color.Purple, fontcolor=Color.White, filled=True
        )
        for node, edges in source_nodes.items():
            if len(edges) == 0:
                self.nodes.remove(node)
                self.nodes.add(node.with_style(style=entrypoint_node_style))

        # fundamental
        fundamental_node_style = ModuleNodeStyle(
            color=Color.Teal, fontcolor=Color.White, filled=True
        )
        for node, edges in dest_nodes.items():
            if len(edges) == 0:
                self.nodes.remove(node)
                self.nodes.add(node.with_style(style=fundamental_node_style))

        # both reference
        both_reference_edge_style = ModuleEdgeStyle(
            color=Color.Red, penwidth=PenWidth.Bold
        )
        for edge in self.edges:
            reverse = edge.build_reverse()
            if reverse in self.edges:
                self.edges.remove(edge)
                self.edges.add(edge.with_style(style=both_reference_edge_style))

    def successors(self, node: ModuleNode) -> List[ModuleNode]:
        return [e.head for e in self.edges if e.tail == node]

    def predecessors(self, node: ModuleNode) -> List[ModuleNode]:
        return [e.tail for e in self.edges if e.head == node]

    def dig(self, node: ModuleNode):
        node_owner = self.find_node_owner(node)
        if not node_owner:
            raise ValueError(f"指定されたモジュール {node.name} が存在しません。")

        next_path_level = node.path_level + 1

        self._dig_successors(node, next_path_level)
        self._dig_predecessors(node, next_path_level)
        self._dig_inner_edge(node, next_path_level)
        self._dig_clustering(node, next_path_level, node_owner)

        self.remove_node(node)

    def _dig_inner_edge(self, node: ModuleNode, next_path_level: int):
        for edge in self.master_graph.find_edges(node):
            self.add_edge(edge.limit_path_level(next_path_level))

    def _dig_clustering(
        self,
        node: ModuleNode,
        next_path_level: int,
        node_owner: Union["Graph", Cluster],
    ):
        cluster = Cluster(node=node)
        for n in self.master_graph.find_nodes(node):
            cluster.add(n.limit_path_level(next_path_level))

        node_owner.add_cluster(cluster)

    def _dig_successors(self, node: ModuleNode, next_path_level: int):
        # 現在のnodeからの接続先エッジを取得
        current_edges = ModuleEdgeCollection(
            [ModuleEdge(node, s) for s in self.successors(node)]
        )

        for new_edge in self.master_graph:
            parent_edge = current_edges.find_parent_edge(new_edge)

            if parent_edge:
                # ノード分解した新しいノードから、ノード分解していないノードへつなぐ
                edge = ModuleEdge(
                    new_edge.tail.limit_path_level(next_path_level), parent_edge.head
                )
                self.add_edge(edge)

    def _dig_predecessors(self, node: ModuleNode, next_path_level: int):
        # 現在のnodeからの接続元エッジを取得
        current_edges = ModuleEdgeCollection(
            [ModuleEdge(p, node) for p in self.predecessors(node)]
        )

        for new_edge in self.master_graph:
            parent_edge = current_edges.find_parent_edge(new_edge)

            if parent_edge:
                # ノード分解した新しいノードから、ノード分解していないノードへつなぐ
                edge = ModuleEdge(
                    parent_edge.tail, new_edge.head.limit_path_level(next_path_level)
                )
                self.add_edge(edge)



import os
from pathlib import Path

import fire

from jig.analyzer.domain.dependency.import_dependency import ImportDependencyCollection
from jig.collector.application import SourceCodeCollector
from jig.collector.domain.source_code.source_code_collection import SourceCodeCollection
from jig.visualizer.application import ModuleDependencyVisualizer


def _collect_source_codes(target_path: str) -> SourceCodeCollection:
    root_path = os.getcwd()

    return SourceCodeCollector(root_path=Path(root_path)).collect(
        target_path=Path(target_path)
    )


def output_dependency_images(target_path, output_dir="output"):
    """
    指定されたディレクトリ以下を解析し、その結果を出力ディレクトリに出力します。
    各Pythonファイルのモジュール名は、実行時のカレントディレクトリをPYTHONPATHとして解釈されます。
    :param target_path: 解析対象のパスを指定します。
    :param output_dir: 解析結果の出力ディレクトリを指定します（デフォルト: output）
    :return:
    """

    for depth in range(1, 8):
        source_codes = _collect_source_codes(target_path=target_path)

        collection = ImportDependencyCollection.build_from_source_code_collection(
            source_codes
        )
        dependencies = collection.build_module_dependencies()

        visualizer = ModuleDependencyVisualizer(dependencies=dependencies)
        visualizer.visualize(depth=depth, output_dir=output_dir)
        visualizer.render_dot_text(depth=depth, output_dir=output_dir)


def main():
    fire.Fire(output_dependency_images)


if __name__ == "__main__":
    main()



import dataclasses
from typing import Union, List

from graphviz import Digraph

from jig.visualizer.module_dependency.domain.model.graph import Graph
from jig.visualizer.module_dependency.domain.model.master_graph import MasterGraph
from jig.visualizer.module_dependency.domain.value.module_edge import (
    ModuleEdge,
    ModuleEdgeCollection,
)
from jig.visualizer.module_dependency.domain.value.module_node import ModuleNode
from jig.visualizer.module_dependency.domain.value.penwidth import Color, PenWidth
from jig.visualizer.module_dependency.presentation.renderer.graph_renderer import (
    GraphRenderer,
)


@dataclasses.dataclass(frozen=True)
class GraphController:
    graph: Graph

    @classmethod
    def sample(cls) -> "GraphController":
        master_graph = MasterGraph(
            edges=ModuleEdgeCollection(
                [ModuleEdge.from_str("A.A", "B.B"), ModuleEdge.from_str("X.A", "Y.B")]
            )
        )

        g = Graph(master_graph=master_graph)
        g.add_edge(ModuleEdge.from_str("A", "B"))
        g.add_edge(ModuleEdge.from_str("X", "Y"))

        return cls(g)

    def remove(self, node_name: str):
        node = ModuleNode.from_str(node_name)
        self.graph.remove_node(node)

    def dig(self, node_name: str):
        node = ModuleNode.from_str(node_name)
        self.graph.dig(node)

    def render(self) -> Digraph:
        renderer = GraphRenderer(self.graph)
        return renderer.render()

    def hide(self, node_name: str):
        node = ModuleNode.from_str(node_name)
        self.graph.hide_node(node)

    def style(
        self,
        node_names: Union[str, List[str]],
        color: str = "black",
        fontcolor: str = "black",
        penwidth: str = "normal",
    ):
        if isinstance(node_names, str):
            nodes = [ModuleNode.from_str(node_names)]
        else:
            nodes = [ModuleNode.from_str(n) for n in node_names]

        color_ = Color(color)
        fontcolor_ = Color(fontcolor)
        penwidth_ = PenWidth(penwidth)

        for node in nodes:
            self.graph.style(
                node=node, color=color_, fontcolor=fontcolor_, penwidth=penwidth_
            )

    def auto_highlight(self):
        self.graph.auto_highlight()



import dataclasses
from typing import Optional, List, Tuple, Dict

from .penwidth import Color, PenWidth
from .module_node import ModuleNode


@dataclasses.dataclass(frozen=True)
class ModuleEdgeStyle:
    color: Color = dataclasses.field(default=Color.Black)
    fontcolor: Color = dataclasses.field(default=Color.Black)
    penwidth: PenWidth = dataclasses.field(default=PenWidth.Normal)
    filled: bool = False
    invisible: bool = False

    def to_dict(self) -> Dict[str, str]:
        return {
            "color": self.color.value,
            "fontcolor": self.fontcolor.value,
            "penwidth": self.penwidth.to_size(self.penwidth),
            "style": self.style,
            "dir": self.dir,
        }

    @property
    def dir(self) -> str:
        # invisible edge の場合には、無向グラフとして振る舞う
        # (style="invisible" としても矢印が残ってしまうので)
        if self.invisible:
            return "none"
        return "directed"

    @property
    def style(self) -> str:
        if self.invisible:
            return "invisible"
        if self.filled:
            return "filled"

        return ""


@dataclasses.dataclass(frozen=True)
class ModuleEdge:
    tail: ModuleNode
    head: ModuleNode
    style: ModuleEdgeStyle = dataclasses.field(
        default_factory=ModuleEdgeStyle, compare=False
    )

    @classmethod
    def from_str(cls, tail: str, head: str) -> "ModuleEdge":
        return cls(tail=ModuleNode.from_str(tail), head=ModuleNode.from_str(head))

    @classmethod
    def build(
        cls, tail: ModuleNode, head: ModuleNode, style: Optional[ModuleEdgeStyle] = None
    ) -> "ModuleEdge":
        return cls(tail=tail, head=head, style=style or ModuleEdgeStyle())

    def belongs_to(self, other: "ModuleEdge") -> bool:
        return self.tail.belongs_to(other.tail) and self.head.belongs_to(other.head)

    def has_node(self, node: ModuleNode) -> bool:
        return node == self.tail or node == self.head

    def is_self_loop(self) -> bool:
        return self.tail == self.head

    def limit_path_level(self, max_path_level: int) -> "ModuleEdge":
        assert max_path_level > 0

        new_tail = self.tail.limit_path_level(max_path_level)
        new_head = self.head.limit_path_level(max_path_level)
        return ModuleEdge(tail=new_tail, head=new_head)

    def __lt__(self, other: "ModuleEdge"):
        if self.head == other.head:
            return self.tail < other.tail

        return self.head < other.head

    def build_reverse(self) -> "ModuleEdge":
        return self.build(tail=self.head, head=self.tail, style=self.style)

    def to_invisible(self) -> "ModuleEdge":
        return self.build(
            tail=self.tail, head=self.head, style=ModuleEdgeStyle(invisible=True)
        )

    def with_style(self, style: ModuleEdgeStyle) -> "ModuleEdge":
        return self.build(tail=self.tail, head=self.head, style=style)


@dataclasses.dataclass(frozen=True)
class ModuleEdgeCollection:
    _edges: List[ModuleEdge] = dataclasses.field(default_factory=list)

    @classmethod
    def from_tuple_list(cls, edges: List[Tuple[str, str]]) -> "ModuleEdgeCollection":
        return cls([ModuleEdge.from_str(tail=e[0], head=e[1]) for e in edges])

    def find_parent_edge(self, edge: ModuleEdge) -> Optional[ModuleEdge]:
        for e in self._edges:
            if edge.belongs_to(e):
                return e
        return None

    def __iter__(self):
        return iter(self._edges)



import dataclasses

from graphviz import Digraph

from jig.visualizer.module_dependency.domain.value.cluster import Cluster


@dataclasses.dataclass(frozen=True)
class ClusterRenderer:
    cluster: Cluster

    @classmethod
    def build(cls, cluster: Cluster) -> "ClusterRenderer":
        return cls(cluster=cluster)

    def render(self) -> Digraph:
        g = Digraph(name=f"cluster_{self.cluster.node.name}")
        g.attr(label=self.cluster.node.name)

        for node in sorted(self.cluster.children):
            g.node(node.name)

        for child_cluster in self.cluster.clusters.values():
            renderer = self.build(child_cluster)
            g.subgraph(renderer.render())

        return g



import dataclasses

from jig.analyzer.domain.dependency.import_dependency import ImportDependencyCollection
from jig.cli.main import _collect_source_codes
from jig.visualizer.module_dependency.domain.model.graph import Graph
from jig.visualizer.module_dependency.domain.model.master_graph import MasterGraph
from jig.visualizer.module_dependency.domain.value.module_edge import ModuleEdge
from jig.visualizer.module_dependency.domain.value.module_node import ModuleNode
from jig.visualizer.module_dependency.presentation.controller.graph_controller import (
    GraphController,
)


@dataclasses.dataclass
class Jig:
    @classmethod
    def analyze_module_dependency(cls, target_path: str) -> GraphController:
        source_codes = _collect_source_codes(target_path=target_path)

        collection = ImportDependencyCollection.build_from_source_code_collection(
            source_codes
        )
        dependencies = collection.build_module_dependencies()
        dependency_tuples = []
        for dependency in dependencies:
            dependency_tuples.append((str(dependency.src), str(dependency.dest)))

        master_graph = MasterGraph.from_tuple_list(dependency_tuples)
        graph = Graph(master_graph=master_graph)

        # トップレベルの依存関係をGraphに追加する
        for dependency in dependencies:
            edge = (
                str(dependency.src.path_in_depth(1)),
                str(dependency.dest.path_in_depth(1)),
            )
            if edge[0] == edge[1]:
                graph.add_node(ModuleNode.from_str(edge[0]))
            else:
                graph.add_edge(ModuleEdge.from_str(edge[0], edge[1]))

        return GraphController(graph=graph)
